// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// include CUDA or emulate relevant types, specific helpers
// ========================================================

#include "hawaii/GPU/CUDA.h"
#include "hawaii/common/error.h"

namespace hawaii {
namespace GPU {

// grid for launching image processing CUDA kernels
dim3 grid( const int  rows,
           const int  cols,
           const dim3 blockDim,
           const uint colsPerThread ) {
	
	// check input
	HAWAII_ERROR_CONDITIONAL( rows < 1 || cols < 1, "Number of rows and columns must be positive." ) ;
	
	// compute next-largest grid to cover all rows and columns
	const uint colThreads = ( cols + colsPerThread-1 ) / colsPerThread ;
	return dim3( ( colThreads + blockDim.x-1 ) / blockDim.x,
	             ( rows       + blockDim.y-1 ) / blockDim.y ) ;
}

} } // namespaces "hawaii::GPU"
