// Copyright (C) 2014 by Karlsruher Institut fuer Technologie (KIT), Institut fuer Mess- und Regelungstechnik
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of arDroneManual/Autonomous.
// 
// arDroneManual/Autonomous is free software: you can redistribute it and/or modify it under the terms of the GNU 
// General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
// option) any later version.
// 
// arDroneManual/Autonomous is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
// 
// You should have received a copy of the GNU General Public License along with arDroneManual/Autonomous. If not, see 
// <http://www.gnu.org/licenses/>.


// off-line test for line detection
// ================================

#include "lineDetection.h"
#include "hawaii/GPU/init.h"
#include "hawaii/common/ostream.h"
#include <opencv2/highgui/highgui.hpp>
#include <iostream>

// helper to handle a pixel's color channels
struct BGR8 { uint8_t b, g, r ; } ;

// entry point
int main() {
	
	// GPU initialization
	hawaii::GPU::init() ; {
		
		// read image from disk
		cv::Mat image = cv::imread( "data/simple/frame0000.jpg" ) ; // STUD_TODO try different images
		
		// perform line detection
		const Coeffs coeffs = detectLine( image ) ;
		
		// print the estimated polynomial model
		if( isnan( coeffs[ 0 ] ) == false ) {
			hawaii::cout << "fitted model: u = " << coeffs[0] << " * v³\n"
			             << "                + " << coeffs[1] << " * v²\n"
			             << "                + " << coeffs[2] << " * v\n"
			             << "                + " << coeffs[3] << " * 1"    << hawaii::endl ;
		}
		
		// keep the visualization image open until a key is pressed
		cv::waitKey() ;
		
	// GPU shutdown
	} hawaii::GPU::reset() ;
	
	// return "0" as in success
	return 0 ;
}

